package wacc

object types {
  import ast._

  sealed trait SemType
  case object ? extends SemType
  enum KnownType extends SemType {
    case Int
    case Bool
    case Char
    case String
    case Array(elem: SemType)
    case Pair(left: SemType, right: SemType)
    case Func(ret: SemType, params: List[SemType])
  }

  object SemType {
    def apply(synType: Type | PairElemType): KnownType = synType match {
      case IntType()    => KnownType.Int
      case BoolType()   => KnownType.Bool
      case CharType()   => KnownType.Char
      case StringType() => KnownType.String
      // For semantic types it is easier to work with recursion rather than a fixed size
      case ArrayType(elemType, dimension) =>
        (0 until dimension).foldLeft(SemType(elemType))((acc, _) => KnownType.Array(acc))
      case PairType(fst, snd) => KnownType.Pair(SemType(fst), SemType(snd))
      case UntypedPairType()  => KnownType.Pair(?, ?)
    }
  }
}
